<?php return array(
    'root' => array(
        'name' => 'icinga/icinga-php-thirdparty',
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'reference' => '09acfc2aa8fd49a6275d45a5fd2526be17332acc',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'brick/math' => array(
            'pretty_version' => '0.14.8',
            'version' => '0.14.8.0',
            'reference' => '63422359a44b7f06cae63c3b429b59e8efcc0629',
            'type' => 'library',
            'install_path' => __DIR__ . '/../brick/math',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/block-react' => array(
            'pretty_version' => 'v1.5.0',
            'version' => '1.5.0.0',
            'reference' => '718b0571a94aa693c6fffc72182e87257ac900f3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/block-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/connection-manager-extra' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'reference' => '6ddf7da4cd75bf5aa54be13eea7be9aad9e97ada',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/connection-manager-extra',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/http-proxy-react' => array(
            'pretty_version' => 'v1.9.0',
            'version' => '1.9.0.0',
            'reference' => 'f3b02835273036a9370ac1c144b55df8e2b98430',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/http-proxy-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/mq-react' => array(
            'pretty_version' => 'v1.7.0',
            'version' => '1.7.0.0',
            'reference' => '0b6b9b097603dd0374a31822021372614df736ea',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/mq-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/redis-protocol' => array(
            'pretty_version' => 'v0.3.2',
            'version' => '0.3.2.0',
            'reference' => '6f565332f5531b7722d1e9c445314b91862f6d6c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-protocol',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/redis-react' => array(
            'pretty_version' => 'v2.8.0',
            'version' => '2.8.0.0',
            'reference' => '84569198dfd5564977d2ae6a32de4beb5a24bdca',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/soap-react' => array(
            'pretty_version' => 'v2.0.0',
            'version' => '2.0.0.0',
            'reference' => 'db2e0ba9b0380ac6c4a34df3f1021d2191d45d45',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/soap-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socket-raw' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => '91e9f619f6769f931454a9882c21ffd7623d06cb',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socket-raw',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/socks-react' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => 'eb3bd2ca5f43fea4b3bc4ef055179e0e6714730d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socks-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/stdio-react' => array(
            'pretty_version' => 'v2.6.0',
            'version' => '2.6.0.0',
            'reference' => 'dfa6c378aabdff718202d4e2453f752c38ea3399',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/stdio-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/term-react' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => '00f297dc597eaee2ebf98af8f27cca5d21d60fa3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/term-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'clue/utf8-react' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'reference' => 'd5cd04d39cb5457aa5df830b7c4b301d2694217e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/utf8-react',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'components/jquery' => array(
            'pretty_version' => 'v3.7.1',
            'version' => '3.7.1.0',
            'reference' => '8edc7785239bb8c2ad2b83302b856a1d61de60e7',
            'type' => 'component',
            'install_path' => __DIR__ . '/../components/jquery',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/collections' => array(
            'pretty_version' => '2.6.0',
            'version' => '2.6.0.0',
            'reference' => '7713da39d8e237f28411d6a616a3dce5e20d5de2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/collections',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'doctrine/deprecations' => array(
            'pretty_version' => '1.1.6',
            'version' => '1.1.6.0',
            'reference' => 'd4fe3e6fd9bb9e72557a19674f44d8ac7db4c6ca',
            'type' => 'library',
            'install_path' => __DIR__ . '/../doctrine/deprecations',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dompdf/dompdf' => array(
            'pretty_version' => 'v3.1.5',
            'version' => '3.1.5.0',
            'reference' => 'f11ead23a8a76d0ff9bbc6c7c8fd7e05ca328496',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dompdf/dompdf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dompdf/php-font-lib' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => 'a6e9a688a2a80016ac080b97be73d3e10c444c9a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dompdf/php-font-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dompdf/php-svg-lib' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => '8259ffb930817e72b1ff1caef5d226501f3dfeb1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dompdf/php-svg-lib',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'dragonmantank/cron-expression' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => 'd61a8a9604ec1f8c3d150d09db6ce98b32675013',
            'type' => 'library',
            'install_path' => __DIR__ . '/../dragonmantank/cron-expression',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'evenement/evenement' => array(
            'pretty_version' => 'v3.0.2',
            'version' => '3.0.2.0',
            'reference' => '0a16b0d71ab13284339abb99d9d2bd813640efbc',
            'type' => 'library',
            'install_path' => __DIR__ . '/../evenement/evenement',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ezyang/htmlpurifier' => array(
            'pretty_version' => 'v4.19.0',
            'version' => '4.19.0.0',
            'reference' => 'b287d2a16aceffbf6e0295559b39662612b77fcf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ezyang/htmlpurifier',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'fig/http-message-util' => array(
            'pretty_version' => '1.1.5',
            'version' => '1.1.5.0',
            'reference' => '9d94dc0154230ac39e5bf89398b324a86f63f765',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fig/http-message-util',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'pretty_version' => '7.10.0',
            'version' => '7.10.0.0',
            'reference' => 'b51ac707cfa420b7bfd4e4d5e510ba8008e822b4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/guzzle',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '2.3.0',
            'version' => '2.3.0.0',
            'reference' => '481557b130ef3790cf82b713667b43030dc9c957',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '2.9.0',
            'version' => '2.9.0.0',
            'reference' => '7d0ed42f28e42d61352a7a79de682e5e67fec884',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'icinga/icinga-php-thirdparty' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'reference' => '09acfc2aa8fd49a6275d45a5fd2526be17332acc',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'icinga/zf1' => array(
            'pretty_version' => 'v2.0.1',
            'version' => '2.0.1.0',
            'reference' => '7acc93188de48501cccadb7f223c0aebeec5bf57',
            'type' => 'library',
            'install_path' => __DIR__ . '/../icinga/zf1',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-color-output' => array(
            'pretty_version' => '3.0.0',
            'version' => '3.0.0.0',
            'reference' => '6c7bf16686cc6a291647fcb87491640a2d5edd20',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-color-output',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-diff' => array(
            'pretty_version' => '6.16.2',
            'version' => '6.16.2.0',
            'reference' => '7f46bcfc582e81769237d0b3f6b8a548efe8799d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-diff',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-mb-string' => array(
            'pretty_version' => '2.0.1',
            'version' => '2.0.1.0',
            'reference' => '8407bfefde47849c9e7c9594e6de2ac85a0f845d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-mb-string',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'jfcherng/php-sequence-matcher' => array(
            'pretty_version' => '4.0.3',
            'version' => '4.0.3.0',
            'reference' => 'd2038ac29627340a7458609072a8ba355e80ec5b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-sequence-matcher',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'masterminds/html5' => array(
            'pretty_version' => '2.10.0',
            'version' => '2.10.0.0',
            'reference' => 'fcf91eb64359852f00d921887b219479b4f21251',
            'type' => 'library',
            'install_path' => __DIR__ . '/../masterminds/html5',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mtdowling/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '^1.0',
            ),
        ),
        'nikic/php-parser' => array(
            'pretty_version' => 'v5.7.0',
            'version' => '5.7.0.0',
            'reference' => 'dca41cd15c2ac9d055ad70dbfd011130757d1f82',
            'type' => 'library',
            'install_path' => __DIR__ . '/../nikic/php-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'parsedown/parsedown' => array(
            'pretty_version' => '1.8.0',
            'version' => '1.8.0.0',
            'reference' => '96baaad00f71ba04d76e45b4620f54d3beabd6f7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../parsedown/parsedown',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'phpstan/phpdoc-parser' => array(
            'pretty_version' => '2.3.2',
            'version' => '2.3.2.0',
            'reference' => 'a004701b11273a26cd7955a61d67a7f1e525a45a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../phpstan/phpdoc-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'predis/predis' => array(
            'pretty_version' => 'v3.4.2',
            'version' => '3.4.2.0',
            'reference' => '2033429520d8997a7815a2485f56abe6d2d0e075',
            'type' => 'library',
            'install_path' => __DIR__ . '/../predis/predis',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/container' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'reference' => 'c71ecc56dfe541dbd90c5360474fbc405f8d5963',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client' => array(
            'pretty_version' => '1.0.3',
            'version' => '1.0.3.0',
            'reference' => 'bb5906edc1c324c9a05aa0873d40117941e5fa90',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-client',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-client-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-factory' => array(
            'pretty_version' => '1.1.0',
            'version' => '1.1.0.0',
            'reference' => '2b4765fddfe3b508ac62f829e852b1501d3f6e8a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-factory',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-factory-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.1',
            'version' => '1.1.0.0',
            'reference' => 'cb6ce4845ce34a8ad9e68117c10ee90a29919eba',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'psr/http-server-handler' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => '84c4fb66179be4caaf8e97bd239203245302e7d4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-server-handler',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/http-server-middleware' => array(
            'pretty_version' => '1.0.2',
            'version' => '1.0.2.0',
            'reference' => 'c1481f747daaa6a0782775cd6a8c26a1bf4a3829',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-server-middleware',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log' => array(
            'pretty_version' => '3.0.2',
            'version' => '3.0.2.0',
            'reference' => 'f16e1d5863e37f8d8c2a01719f5b34baa2b714d3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'radebatz/type-info-extras' => array(
            'pretty_version' => '1.0.7',
            'version' => '1.0.7.0',
            'reference' => '95a524a74a61648b44e355cb33d38db4b17ef5ce',
            'type' => 'library',
            'install_path' => __DIR__ . '/../radebatz/type-info-extras',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/collection' => array(
            'pretty_version' => '2.1.1',
            'version' => '2.1.1.0',
            'reference' => '344572933ad0181accbf4ba763e85a0306a8c5e2',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/collection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '4.9.2',
            'version' => '4.9.2.0',
            'reference' => '8429c78ca35a09f27565311b98101e2826affde0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/async' => array(
            'pretty_version' => 'v4.3.0',
            'version' => '4.3.0.0',
            'reference' => '635d50e30844a484495713e8cb8d9e079c0008a5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/async',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/cache' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'reference' => 'd47c472b64aa5608225f47965a484b75c7817d5b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/cache',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/child-process' => array(
            'pretty_version' => 'v0.6.7',
            'version' => '0.6.7.0',
            'reference' => '970f0e71945556422ee4570ccbabaedc3cf04ad3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/child-process',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/datagram' => array(
            'pretty_version' => 'v1.10.0',
            'version' => '1.10.0.0',
            'reference' => '9236e1f5a67a6029be17d551e9858c487836c301',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/datagram',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/dns' => array(
            'pretty_version' => 'v1.14.0',
            'version' => '1.14.0.0',
            'reference' => '7562c05391f42701c1fccf189c8225fece1cd7c3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/dns',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/event-loop' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'reference' => 'ba276bda6083df7e0050fd9b33f66ad7a4ac747a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/event-loop',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/http' => array(
            'pretty_version' => 'v1.11.0',
            'version' => '1.11.0.0',
            'reference' => '8db02de41dcca82037367f67a2d4be365b1c4db9',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/http',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise' => array(
            'pretty_version' => 'v2.11.0',
            'version' => '2.11.0.0',
            'reference' => '1a8460931ea36dc5c76838fec5734d55c88c6831',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise-stream' => array(
            'pretty_version' => 'v1.7.0',
            'version' => '1.7.0.0',
            'reference' => '5c7ec3450f558deb779742e33967d837e2db7871',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-stream',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/promise-timer' => array(
            'pretty_version' => 'v1.11.0',
            'version' => '1.11.0.0',
            'reference' => '4f70306ed66b8b44768941ca7f142092600fafc1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-timer',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/socket' => array(
            'pretty_version' => 'v1.17.0',
            'version' => '1.17.0.0',
            'reference' => 'ef5b17b81f6f60504c539313f94f2d826c5faa08',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/socket',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'react/stream' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'reference' => '1e5b0acb8fe55143b5b426817155190eb6f5b18d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/stream',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.9.2',
            ),
        ),
        'ringcentral/psr7' => array(
            'pretty_version' => '1.3.0',
            'version' => '1.3.0.0',
            'reference' => '360faaec4b563958b673fb52bbe94e37f14bc686',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ringcentral/psr7',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'sabberworm/php-css-parser' => array(
            'pretty_version' => 'v9.3.0',
            'version' => '9.3.0.0',
            'reference' => '88dbd0f7f91abbfe4402d0a3071e9ff4d81ed949',
            'type' => 'library',
            'install_path' => __DIR__ . '/../sabberworm/php-css-parser',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'simshaun/recurr' => array(
            'pretty_version' => 'v5.0.3',
            'version' => '5.0.3.0',
            'reference' => '7b136768d64f257065e38a804ee6d2f9af6ba6d1',
            'type' => 'library',
            'install_path' => __DIR__ . '/../simshaun/recurr',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/deprecation-contracts' => array(
            'pretty_version' => 'v3.6.0',
            'version' => '3.6.0.0',
            'reference' => '63afe740e99a13ba87ec199bb07bbdee937a5b62',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/deprecation-contracts',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/finder' => array(
            'pretty_version' => 'v7.4.6',
            'version' => '7.4.6.0',
            'reference' => '8655bf1076b7a3a346cb11413ffdabff50c7ffcf',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/finder',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => 'a3cc8b044a6ea513310cbd48ef7333b384945638',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php83' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => '17f6f9a6b1735c0f163024d959f700cfbc5155e5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php83',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php84' => array(
            'pretty_version' => 'v1.33.0',
            'version' => '1.33.0.0',
            'reference' => 'd8ced4d875142b6a7426000426b8abc631d6b191',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php84',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/type-info' => array(
            'pretty_version' => 'v7.4.7',
            'version' => '7.4.7.0',
            'reference' => '31f1e40cbf7851c7354281c90eb1b352c4cb8269',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/type-info',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'symfony/yaml' => array(
            'pretty_version' => 'v7.4.6',
            'version' => '7.4.6.0',
            'reference' => '58751048de17bae71c5aa0d13cb19d79bca26391',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/yaml',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'tedivm/jshrink' => array(
            'pretty_version' => 'v1.8.1',
            'version' => '1.8.1.0',
            'reference' => 'f76454d4c48ddae6354a2f0eeb16633d4e755c9a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../tedivm/jshrink',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'thecodingmachine/safe' => array(
            'pretty_version' => 'v3.4.0',
            'version' => '3.4.0.0',
            'reference' => '705683a25bacf0d4860c7dea4d7947bfd09eea19',
            'type' => 'library',
            'install_path' => __DIR__ . '/../thecodingmachine/safe',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'wikimedia/less.php' => array(
            'pretty_version' => 'v3.2.1',
            'version' => '3.2.1.0',
            'reference' => '0d5b30ba792bdbf8991a646fc9c30561b38a5559',
            'type' => 'library',
            'install_path' => __DIR__ . '/../wikimedia/less.php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'zircote/swagger-php' => array(
            'pretty_version' => '6.0.6',
            'version' => '6.0.6.0',
            'reference' => '9447c1f45b5ae93185caea9a0c8e298399188a25',
            'type' => 'library',
            'install_path' => __DIR__ . '/../zircote/swagger-php',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
