<?php declare(strict_types=1);

namespace PhpParser\Node\Scalar;

require __DIR__ . '/Float_.php';

if (false) {
    /**
     * For classmap-authoritative support.
     *
     * @deprecated use \PhpParser\Node\Scalar\Float_ instead.
     */
    class DNumber extends Float_ {
    }
}
